#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <asm-arm/arch-sa1100/hijack.h>

#include "vfdlib.c"

#define CHECK(rcode)	if ((rc = (rcode)) < 0) errorExit((rc))
#define TEXTSTRING "This is sweet!"
#define TEXTSIZE VFDFONT_MEDIUM

int fd;		// display fd
int rc;		// hijack ioctl return codes for errorExit
unsigned char screenbuf[EMPEG_SCREEN_BYTES] = {0,};	// display buffer
unsigned long buttons[4] = {4,		// buttons our program will "hijack"
		IR_KNOB_PRESSED,	// from the player
		IR_KNOB_LEFT,
		IR_KNOB_RIGHT};

// prototypes
void moveText(char *text, int width);
void errorExit(int code);

int main()
{
	char *hijackMenus[] = {"scroller", NULL};	// title(s) of our program

	int textWidth;
	char *scrollText = TEXTSTRING;

	// check that the text string will fit
	if ((textWidth = vfdlib_getTextWidth(scrollText, TEXTSIZE)) >= VFD_WIDTH) {
		fprintf(stderr, "Text string too long, exiting...\n");
		exit(1);
	}

	fd = open("/dev/display",O_RDWR);

	while (1) {
		// bind to hijack's menu and wait to be selected
		CHECK(ioctl(fd,EMPEG_HIJACK_WAITMENU,hijackMenus));
		// we have a valid return code, start the scroller
		moveText(scrollText, textWidth);
		// when the scroller is done we'll go back to waiting
		// for a menu selection
	}

	return 0;
}

void moveText(char *text, int width) {
	// start with the text centered in the middle of the display
	int textX = (VFD_WIDTH - width) / 2;
	int textY = (VFD_HEIGHT - vfdlib_getTextHeight(TEXTSIZE)) / 2;

	char cContinue = 1;
	unsigned long ulButtonCode;

	// take over the buttons in the button array
	CHECK(ioctl(fd,EMPEG_HIJACK_BINDBUTTONS, buttons));

	while (cContinue) {
		// black the display
		vfdlib_clear(screenbuf, VFDSHADE_BLACK);
		// place the text in the new position
		vfdlib_drawText(screenbuf, text, textX, textY, TEXTSIZE, VFDSHADE_BRIGHT);
		// write the display buffer to the screen
		CHECK(ioctl(fd,EMPEG_HIJACK_DISPWRITE, screenbuf));
		// wait for a button press
		CHECK(ioctl(fd,EMPEG_HIJACK_WAITBUTTONS, &ulButtonCode));
		switch (ulButtonCode) {
			case IR_KNOB_LEFT:
				if (textX > 1)
					textX -= 2;
				break;
			case IR_KNOB_RIGHT:
				if (textX < (VFD_WIDTH - width - 1))
					textX += 2;
				break;
			case IR_KNOB_PRESSED:
				// exit this loop
				cContinue = 0;
				break;
			default:
				break;
		}
	}
}

void errorExit(int code) {
	// hijack must have returned an invalid return code
	char message[15];
	snprintf(message, 14, "RC = %d", code);
	perror(message);
	exit(1);
}
